"""auxjad
======

Auxiliary functions and classes for Abjad 3.1. All classes and functions have a
`__doc__` attribute with usage instructions.

Documentation is available at https://gilbertohasnofb.github.io/auxjad-docs/.
A pdf version of the documentation is also available in the `docs` directory.

Bugs can be reported to https://github.com/gilbertohasnofb/auxjad/issues.

This library is published under the MIT License.
"""

from .tools import *

__author__ = "Gilberto Agostinho <gilbertohasnofb@gmail.com>"
__version__ = "0.4.0"
